import os
import sys
import platform
from collections import defaultdict

if sys.version_info < (3, 4):
    print("You need at least Python 3.4 for this application!")
    sys.exit(1)

try:
    from setuptools import setup, Extension
    from setuptools.command.build_ext import build_ext as _build_ext
except ImportError:
    print("Could not find setuptools")
    print("Try installing them with pip install setuptools")
    sys.exit(1)

from distutils import ccompiler
import src.urh.version as version

if sys.platform == "win32":
    OPEN_MP_FLAG = "-openmp"
elif sys.platform == "darwin":
    OPEN_MP_FLAG = ""  # no OpenMP support in default Mac OSX compiler
else:
    OPEN_MP_FLAG = "-fopenmp"

COMPILER_DIRECTIVES = {'language_level': 3,
                       'cdivision': True,
                       'wraparound': False,
                       'boundscheck': False,
                       'initializedcheck': False,
                       }

UI_SUBDIRS = ("actions", "delegates", "views")
PLUGINS = [path for path in os.listdir("src/urh/plugins") if os.path.isdir(os.path.join("src/urh/plugins", path))]
URH_DIR = "urh"

try:
    import Cython.Build
except ImportError:
    USE_CYTHON = False
else:
    USE_CYTHON = True
EXT = '.pyx' if USE_CYTHON else '.cpp'


class build_ext(_build_ext):
    def finalize_options(self):
        print("Finalizing options")
        _build_ext.finalize_options(self)
        # Prevent numpy from thinking it is still in its setup process:
        __builtins__.__NUMPY_SETUP__ = False
        import numpy
        self.include_dirs.append(numpy.get_include())


def get_packages():
    packages = [URH_DIR]
    separator = os.path.normpath("/")
    for dirpath, dirnames, filenames in os.walk(os.path.join("./src/", URH_DIR)):
        package_path = os.path.relpath(dirpath, os.path.join("./src/", URH_DIR)).replace(separator, ".")
        if len(package_path) > 1:
            packages.append(URH_DIR + "." + package_path)

    return packages


def get_package_data():
    package_data = {"urh.cythonext": ["*.cpp", "*.pyx"]}
    for plugin in PLUGINS:
        package_data["urh.plugins." + plugin] = ['*.ui', "*.txt"]

    is_release = os.path.isfile("/tmp/urh_releasing")  # make sure precompiled binding are uploaded to PyPi

    package_data["urh.dev.native.lib"] = ["*.cpp", "*.pyx", "*.pxd"]

    # Bundle headers
    package_data["urh.dev.native.includes"] = ["*.h"]
    package_data["urh.dev.native.includes.libhackrf"] = ["*.h"]

    if sys.platform == "win32" or is_release:
        # we use precompiled device backends on windows
        package_data["urh.dev.native.lib.win"] = ["*"]

    return package_data


def get_ext_modules():
    filenames = [os.path.splitext(f)[0] for f in os.listdir("src/urh/cythonext") if f.endswith(EXT)]

    extensions = [Extension("urh.cythonext." + f, ["src/urh/cythonext/" + f + EXT],
                            extra_compile_args=[OPEN_MP_FLAG],
                            extra_link_args=[OPEN_MP_FLAG],
                            language="c++") for f in filenames]

    return extensions


def __get_device_extension(dev_name: str, libraries: list, library_dirs: list, include_dirs: list, use_cython=False):
    file_ext = "pyx" if use_cython else "cpp"
    return Extension("urh.dev.native.lib." + dev_name, ["src/urh/dev/native/lib/{0}.{1}".format(dev_name, file_ext)],
                     libraries=libraries, library_dirs=library_dirs,
                     include_dirs=include_dirs, language="c++")


def get_device_modules():
    include_dir = os.path.realpath(os.path.join(os.curdir, "src/urh/dev/native/includes"))
    devices = {
        "hackrf": {"lib": "hackrf", "test_function": "hackrf_init"},
        "rtlsdr": {"lib": "rtlsdr", "test_function": "rtlsdr_set_tuner_bandwidth"},
    }

    fallbacks = {
        "rtlsdr": {"lib": "rtlsdr", "test_function": "rtlsdr_get_device_name"}
    }

    if sys.platform == "win32":
        if platform.architecture()[0] != "64bit":
            return []  # only 64 bit python supported for native device backends

        result = []
        lib_dir = os.path.realpath(os.path.join(os.curdir, "src/urh/dev/native/lib/win"))
        for dev_name, params in devices.items():
            result.append(__get_device_extension(dev_name, [params["lib"]], [lib_dir], [include_dir]))

        return result

    if sys.platform == "darwin":
        # On Mac OS X clang is by default not smart enough to search in the lib dir
        # see: https://github.com/jopohl/urh/issues/173
        library_dirs = ["/usr/local/lib"]
    else:
        library_dirs = []

    result = []

    # None = automatic (depending on lib is installed)
    # 1 = install extension always
    # 0 = Do not install extension
    build_device_extensions = defaultdict(lambda: None)

    for dev_name in devices:
        with_option = "--with-" + dev_name
        without_option = "--without-" + dev_name

        if with_option in sys.argv and without_option in sys.argv:
            print("ambiguous options for " + dev_name)
            sys.exit(1)
        elif without_option in sys.argv:
            build_device_extensions[dev_name] = 0
            sys.argv.remove(without_option)
        elif with_option in sys.argv:
            build_device_extensions[dev_name] = 1
            sys.argv.remove(with_option)

    script_dir = os.path.realpath(os.path.dirname(__file__))
    sys.path.append(os.path.realpath(os.path.join(script_dir, "src", "urh", "dev", "native", "lib")))

    compiler = ccompiler.new_compiler()
    for dev_name, params in devices.items():
        if build_device_extensions[dev_name] == 0:
            print("\nSkipping native {0} support\n".format(dev_name))
            continue
        if build_device_extensions[dev_name] == 1:
            print("\nEnforcing native {0} support\n".format(dev_name))
            result.append(__get_device_extension(dev_name, [params["lib"]], library_dirs, [include_dir], USE_CYTHON))
            continue

        if compiler.has_function(params["test_function"], libraries=(params["lib"],), library_dirs=library_dirs,
                                 include_dirs=[include_dir]):
            print("\nFound {0} lib. Will compile with native {1} support\n".format(params["lib"], dev_name))
            result.append(__get_device_extension(dev_name, [params["lib"]], library_dirs, [include_dir], USE_CYTHON))
        elif dev_name in fallbacks:
            print("Trying fallback for {0}".format(dev_name))
            params = fallbacks[dev_name]
            dev_name += "_fallback"
            if compiler.has_function(params["test_function"], libraries=(params["lib"],), library_dirs=library_dirs,
                                     include_dirs=[include_dir]):
                print("\nFound fallback. Will compile with native {0} support\n".format(dev_name))
                result.append(
                    __get_device_extension(dev_name, [params["lib"]], library_dirs, [include_dir], USE_CYTHON))
        else:
            print("\nSkipping native support for {1}\n".format(params["lib"], dev_name))
        try:
            os.remove("a.out")  # Temp file for checking
        except OSError:
            pass

    return result


def read_long_description():
    try:
        import pypandoc
        return pypandoc.convert('README.md', 'rst')
    except(IOError, ImportError, RuntimeError):
        return ""


# import generate_ui
# generate_ui.gen # pyuic5 is not included in all python3-pyqt5 packages (e.g. ubuntu), therefore do not regenerate UI here

install_requires = ["numpy", "psutil", "pyzmq"]
try:
    import PyQt5
except ImportError:
    install_requires.append("pyqt5")

if sys.version_info < (3, 4):
    install_requires.append('enum34')

extensions = get_ext_modules() + get_device_modules()

if USE_CYTHON:
    from Cython.Build import cythonize

    extensions = cythonize(extensions, compiler_directives=COMPILER_DIRECTIVES)

setup(
    name="urh",
    version=version.VERSION,
    description="Universal Radio Hacker: investigate wireless protocols like a boss",
    long_description=read_long_description(),
    author="Johannes Pohl",
    author_email="Johannes.Pohl90@gmail.com",
    package_dir={"": "src"},
    package_data=get_package_data(),
    url="https://github.com/jopohl/urh",
    license="Apache License 2.0",
    download_url="https://github.com/jopohl/urh/tarball/v" + str(version.VERSION),
    install_requires=install_requires,
    setup_requires=['numpy'],
    packages=get_packages(),
    ext_modules=extensions,
    cmdclass={'build_ext': build_ext},
    zip_safe=False,
    entry_points={
        'console_scripts': [
            'urh = urh.main:main',
        ]}
)

# python setup.py sdist --> Source distribution
# python setup.py bdist --> Vorkompiliertes Package https://docs.python.org/3/distutils/builtdist.html



from PyQt5.QtWidgets import QTableView

from urh.models.LabelValueTableModel import LabelValueTableModel
from urh.signalprocessing.ProtocoLabel import ProtocolLabel
from urh.ui.delegates.ComboBoxDelegate import ComboBoxDelegate


class LabelValueTableView(QTableView):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setItemDelegateForColumn(1, ComboBoxDelegate(ProtocolLabel.DISPLAY_FORMATS, parent=self))
        self.setEditTriggers(QTableView.AllEditTriggers)

    def model(self) -> LabelValueTableModel:
        return super().model()



from collections import defaultdict

import numpy as np
from urh import constants
from urh.awre.CommonRange import CommonRange
from urh.cythonext import util
from urh.awre.components.Component import Component
from urh.signalprocessing.MessageType import MessageType


class Address(Component):
    MIN_ADDRESS_LENGTH = 8  # Address should be at least one byte

    def __init__(self, fieldtypes, xor_matrix, priority=2, predecessors=None, enabled=True, backend=None, messagetypes=None):
        super().__init__(priority, predecessors, enabled, backend, messagetypes)
        self.xor_matrix = xor_matrix

        self.dst_field_type = next((ft for ft in fieldtypes if ft.function == ft.Function.DST_ADDRESS), None)
        self.src_field_type = next((ft for ft in fieldtypes if ft.function == ft.Function.SRC_ADDRESS), None)

        self.dst_field_name = self.dst_field_type.caption if self.dst_field_type else "DST address"
        self.src_field_name = self.src_field_type.caption if self.src_field_type else "SRC address"

    def _py_find_field(self, messages, verbose=False):
        """

        :type messages: list of urh.signalprocessing.Message.Message
        :return:
        """
        msg_indices_per_participant = defaultdict(list)
        """:type : dict[urh.signalprocessing.Participant.Participant, list[int]] """

        for i, msg in enumerate(messages):
            msg_indices_per_participant[msg.participant].append(i)


        # Cluster participants
        equal_ranges_per_participant = defaultdict(list)
        """:type : dict[urh.signalprocessing.Participant.Participant, list[CommonRange]] """

        alignment = 8

        # Step 1: Find equal ranges for participants by evaluating the XOR matrix participant wise
        for participant, participant_msg_indices in msg_indices_per_participant.items():
            for i, msg_index in enumerate(participant_msg_indices):
                msg = messages[msg_index]
                bitvector_str = msg.decoded_bits_str

                for other_index in participant_msg_indices[i+1:]:
                    other_msg = messages[other_index]
                    xor_vec = self.xor_matrix[msg_index, other_index][self.xor_matrix[msg_index, other_index] != -1] # -1 = End of Vector

                    # addresses are searched across message types, as we assume them to be in almost every message
                    # therefore we need to consider message types of both messages we compare and ignore already labeled areas
                    unlabeled_ranges = msg.message_type.unlabeled_ranges_with_other_mt(other_msg.message_type)
                    for rng_start, rng_end in unlabeled_ranges:
                        start = 0
                        # The last 1 marks end of sequence, and prevents swallowing long zero sequences at the end
                        cmp_vector = np.append(xor_vec[rng_start:rng_end], 1)
                        for end in np.where(cmp_vector == 1)[0]:
                            if end - start >= self.MIN_ADDRESS_LENGTH:
                                equal_range_start = alignment * ((rng_start + start) // alignment)
                                equal_range_end = alignment * ((rng_start + end) // alignment)
                                bits = bitvector_str[equal_range_start:equal_range_end]

                                # Did we already found this range?
                                cr = next((cr for cr in equal_ranges_per_participant[participant] if
                                          cr.start == equal_range_start and cr.end == equal_range_end
                                          and cr.bits == bits), None)

                                # If not: Create it
                                if cr is None:
                                    cr = CommonRange(equal_range_start, equal_range_end, bits)
                                    equal_ranges_per_participant[participant].append(cr)

                                cr.messages.add(msg_index)
                                cr.messages.add(other_index)

                            start = end + alignment

        if verbose:
            print(constants.color.BOLD + "Result after Step 1" +constants.color.END)
            self.__print_ranges(equal_ranges_per_participant)

        # Step 2: Now we want to find our address candidates.
        # We do this by weighting them in order of LCS they share with each other
        scored_candidates = self.find_candidates([cr for crl in equal_ranges_per_participant.values() for cr in crl])
        """:type : dict[str, int] """

        try:
            highscored = next(self.choose_candidate_pair(scored_candidates))
            assert len(highscored[0]) == len(highscored[1])
        except (StopIteration, AssertionError):
            return

        if verbose:
            print(scored_candidates)
            print(sorted(scored_candidates, key=scored_candidates.get, reverse=True))

        # Now get the common_ranges we need
        scored_candidates_per_participant = defaultdict(list)
        """:type : dict[urh.signalprocessing.Participant.Participant, list[CommonRange]] """

        for participant, ranges in equal_ranges_per_participant.items():
            for equal_range in ranges:
                for h in highscored:
                    rng = equal_range.pos_of_hex(h)
                    if rng is not None:
                        start, end = rng
                        bits = equal_range.bits[start:end]
                        rel_start = equal_range.start + start
                        rel_end = rel_start + (end - start)
                        cr = next((cr for cr in scored_candidates_per_participant[participant] if cr.start == rel_start
                                                                                               and cr.end == rel_end and
                                                                                               cr.bits == bits), None)
                        if cr is None:
                            cr = CommonRange(rel_start, rel_end, bits)
                            scored_candidates_per_participant[participant].append(cr)

                        cr.messages.update(equal_range.messages)

        # Now we have the highscored ranges per participant
        # If there is a crossmatch of the ranges we are good and found the addresses!
        # We have something like:
        #
        # Participant: Alice (A):                               Participant: Bob (B):
        # =======================                               =====================
        #
        # Range	   Value     Messages                           Range	   Value     Messages
        # -----    -----     --------                           -----      -----     --------
        # 72-96    1b6033    {1, 5, 9, 13, 17, 20}              72-96      78e289    {11, 3, 15, 7}
        # 88-112   1b6033    {2, 6, 10, 14, 18}                 88-112     78e289    {4, 8, 12, 16, 19}
        # 112-136  78e289    {2, 6, 10, 14, 18}                 112-136    1b6033    {0, 4, 8, 12, 16, 19}
        #

        # If the value doubles for the same participant in other range, then we need to create a new message type
        # We consider the default case (=default message type) to have addresses followed by each other
        # Furthermore, we assume if there is only one address per message type, it is the destination address
        clusters = {"default": defaultdict(set), "ack": defaultdict(set)}
        """:type: dict[str, dict[tuple[int.int],set[int]]]"""

        all_candidates = [cr for crl in scored_candidates_per_participant.values() for cr in crl]
        # Check for crossmatch and cluster in together and splitted addresses
        # Perform a merge by only saving the ranges and applying messages
        for candidate in sorted(all_candidates):
            if any(c.start == candidate.start and c.end == candidate.end and c.bits != candidate.bits for c in all_candidates):
                # Crossmatch! This is a address
                if any(c.start == candidate.end or c.end == candidate.start for c in all_candidates):
                     clusters["default"][(candidate.start, candidate.end)].update(candidate.messages)
                else:
                    clusters["ack"][(candidate.start, candidate.end)].update(candidate.messages)

        msg_clusters =  {cname: set(i for s in ranges.values() for i in s) for cname, ranges in clusters.items()}

        # If there are no addresses in default message type prevent evaluating everything as ACK
        if not msg_clusters["default"]:
            msg_clusters["ack"] = set()
            scored_candidates_per_participant.clear()

        self.assign_messagetypes(messages, msg_clusters)

        # Now try to find the addresses of the participants to separate SRC and DST address later
        self.assign_participant_addresses(messages, list(scored_candidates_per_participant.keys()), highscored)

        for participant, ranges in scored_candidates_per_participant.items():
            for rng in ranges:
                for msg_index in rng.messages:
                    msg = messages[msg_index]

                    if msg.message_type.name == "ack":
                       field_type = self.dst_field_type
                       name = self.dst_field_name
                    elif msg.participant:
                        if rng.hex_value == msg.participant.address_hex:
                            name = self.src_field_name
                            field_type = self.src_field_type
                        else:
                            name = self.dst_field_name
                            field_type = self.dst_field_type
                    else:
                        name = "Address"
                        field_type = None

                    if not any(lbl.name == name and lbl.auto_created for lbl in msg.message_type):
                        msg.message_type.add_protocol_label(rng.start, rng.end - 1, name=name,
                                                            auto_created=True, type=field_type)


    @staticmethod
    def find_candidates(candidates):
        """
        Find candidate addresses using LCS algorithm
        perform a scoring based on how often a candidate appears in a longer candidate

        Input is something like
        ------------------------
        ['1b6033', '1b6033fd57', '701b603378e289', '20701b603378e289000c62',
        '1b603300', '78e289757e', '7078e2891b6033000000', '207078e2891b6033000000']

        Output like
        -----------
        {'1b6033': 18, '1b6033fd57': 1, '701b603378e289': 2, '207078e2891b6033000000': 1,
        '57': 1, '7078e2891b6033000000': 2, '78e289757e': 1, '20701b603378e289000c62': 1,
        '78e289': 4, '1b603300': 3}

        :type candidates: list of CommonRange
        :return:
        """

        result = defaultdict(int)
        for i, c_i in enumerate(candidates):
            for j in range(i, len(candidates)):
                lcs = util.longest_common_substring(c_i.hex_value, candidates[j].hex_value)
                if lcs:
                    result[lcs] += 1

        return result

    @staticmethod
    def choose_candidate_pair(candidates):
        """
        Choose a pair of address candidates ensuring they have the same length and starting with the highest scored ones

        :type candidates: dict[str, int]
        :param candidates: Count how often the longest common substrings appeared in the messages
        :return:
        """
        highscored = sorted(candidates, key=candidates.get, reverse=True)
        for i, h_i in enumerate(highscored):
            for h_j in highscored[i+1:]:
                if len(h_i) == len(h_j):
                    yield (h_i, h_j)

    @staticmethod
    def assign_participant_addresses(messages, participants, hex_addresses):
        """

        :type participants: list[urh.signalprocessing.Participant.Participant]
        :type hex_addresses: tuple[str]
        :type messages: list[urh.signalprocessing.Message.Message]
        :return:
        """
        try:
            participants.remove(None)
        except ValueError:
            pass

        if len(participants) != len(hex_addresses):
            return

        if len(participants) == 0:
            return #  No chance


        score = {p: {addr: 0 for addr in hex_addresses} for p in participants}

        for i in range(1, len(messages)):
            msg = messages[i]
            prev_msg = messages[i-1]

            if msg.message_type.name == "ack":
                addr = next(addr for addr in hex_addresses if addr in msg.decoded_hex_str)
                if addr in prev_msg.decoded_hex_str:
                    score[prev_msg.participant][addr] += 1

        for p in participants:
            p.address_hex = max(score[p], key=score[p].get)

    def __print_clustered(self, clustered_addresses):
        for bl in sorted(clustered_addresses):
            print(constants.color.BOLD + "Byte length " + str(bl) + constants.color.END)
            for (start, end), bits in sorted(clustered_addresses[bl].items()):
                print(start, end, bits)

    def __print_ranges(self, equal_ranges_per_participant):
        for parti in sorted(equal_ranges_per_participant):
            if parti is None:
                continue

            print("\n" + constants.color.UNDERLINE + str(parti.name) + " (" + parti.shortname+ ")" + constants.color.END)
            address1 = "000110110110000000110011"
            address2 = "011110001110001010001001"

            assert len(address1) % 8 == 0
            assert len(address2) % 8 == 0

            print("address1", constants.color.BLUE, address1 + " (" +hex(int("".join(map(str, address1)), 2)) +")", constants.color.END)
            print("address2", constants.color.GREEN, address2 + " (" + hex(int("".join(map(str, address2)), 2)) + ")",
                  constants.color.END)

            print()

            for common_range in sorted(equal_ranges_per_participant[parti]):
                assert isinstance(common_range, CommonRange)
                bits_str = common_range.bits
                format_start = ""
                if address1 in bits_str and address2 not in bits_str:
                    format_start = constants.color.BLUE
                if address2 in bits_str and address1 not in bits_str:
                    format_start = constants.color.GREEN
                if address1 in bits_str and address2 in bits_str:
                    format_start = constants.color.RED + constants.color.BOLD

                # For Bob the adress 1b60330 is found to be 0x8db0198000 which is correct,
                # as it starts with a leading 1 in all messages.
                # This is the last Bit of e0003 (Broadcast) or 78e289  (Other address)
                # Code to verify: hex(int("1000"+bin(int("1b6033",16))[2:]+"000",2))
                # Therefore we need to check for partial bits inside the address candidates to be sure we find the correct ones
                occurences = len(common_range.messages)
                print(common_range.start, common_range.end,
                      "({})\t".format(occurences),
                      format_start + common_range.hex_value + "\033[0m", common_range.byte_len,
                      bits_str, "(" + ",".join(map(str, common_range.messages)) + ")")





import locale


# locale.setlocale(locale.LC_ALL, '')
from urh.util.Logger import logger


class Formatter:
    @staticmethod
    def local_decimal_seperator():
        return locale.localeconv()["decimal_point"]

    @staticmethod
    def science_time(time_in_seconds: float, decimals=2, append_seconds=True, remove_spaces=False) -> str:
        if time_in_seconds < 1e-6:
            suffix = "n"
            value = time_in_seconds * 1e9
        elif time_in_seconds < 1e-3:
            suffix = "µ"
            value = time_in_seconds * 1e6
        elif time_in_seconds < 1:
            suffix = "m"
            value = time_in_seconds * 1e3
        else:
            suffix = ""
            value = time_in_seconds

        result = locale.format_string("%.{0}f ".format(decimals) + suffix, value)
        if append_seconds:
            result += "s"
        if remove_spaces:
            result = result.replace(" ", "")

        return result

    @staticmethod
    def big_value_with_suffix(value: float, decimals=3) -> str:
        fmt_str = "%.{0:d}f".format(decimals)
        if abs(value) >= 1e9:
            return locale.format_string(fmt_str+"G", value / 1e9)
        elif abs(value) >= 1e6:
            return locale.format_string(fmt_str+"M", value / 1e6)
        elif abs(value) >= 1e3:
            return locale.format_string(fmt_str+"K", value / 1e3)
        else:
            return locale.format_string(fmt_str, value)


    @staticmethod
    def str2val(str_val, dtype, default=0):
        try:
            return dtype(str_val)
        except (ValueError, TypeError):
            logger.warning("The {0} is not a valid {1}, assuming {2}".format(str_val, str(dtype), str(default)))
            return default



# -*- coding: utf-8 -*-

#
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_DialogOptions(object):
    def setupUi(self, DialogOptions):
        DialogOptions.setObjectName("DialogOptions")
        DialogOptions.resize(953, 653)
        self.horizontalLayout = QtWidgets.QHBoxLayout(DialogOptions)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.tabWidget = QtWidgets.QTabWidget(DialogOptions)
        self.tabWidget.setObjectName("tabWidget")
        self.tabInterpretation = QtWidgets.QWidget()
        self.tabInterpretation.setObjectName("tabInterpretation")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.tabInterpretation)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.groupBox = QtWidgets.QGroupBox(self.tabInterpretation)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setWordWrap(True)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.label_2 = QtWidgets.QLabel(self.groupBox)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 1, 0, 1, 1)
        self.spinBoxSymbolTreshold = QtWidgets.QSpinBox(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.spinBoxSymbolTreshold.sizePolicy().hasHeightForWidth())
        self.spinBoxSymbolTreshold.setSizePolicy(sizePolicy)
        self.spinBoxSymbolTreshold.setMaximum(50)
        self.spinBoxSymbolTreshold.setObjectName("spinBoxSymbolTreshold")
        self.gridLayout.addWidget(self.spinBoxSymbolTreshold, 1, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.groupBox)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 1, 2, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.groupBox)
        self.label_4.setObjectName("label_4")
        self.gridLayout.addWidget(self.label_4, 2, 0, 1, 1)
        self.lSymbolLength = QtWidgets.QLabel(self.groupBox)
        self.lSymbolLength.setObjectName("lSymbolLength")
        self.gridLayout.addWidget(self.lSymbolLength, 2, 1, 1, 1)
        self.label_6 = QtWidgets.QLabel(self.groupBox)
        self.label_6.setObjectName("label_6")
        self.gridLayout.addWidget(self.label_6, 2, 2, 1, 1)
        self.chkBoxEnableSymbols = QtWidgets.QCheckBox(self.groupBox)
        self.chkBoxEnableSymbols.setObjectName("chkBoxEnableSymbols")
        self.gridLayout.addWidget(self.chkBoxEnableSymbols, 0, 0, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.lExplanation = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.lExplanation.setFont(font)
        self.lExplanation.setWordWrap(True)
        self.lExplanation.setObjectName("lExplanation")
        self.verticalLayout.addWidget(self.lExplanation)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.verticalLayout_2.addWidget(self.groupBox)
        self.tabWidget.addTab(self.tabInterpretation, "")
        self.tabGeneration = QtWidgets.QWidget()
        self.tabGeneration.setObjectName("tabGeneration")
        self.layoutWidget = QtWidgets.QWidget(self.tabGeneration)
        self.layoutWidget.setGeometry(QtCore.QRect(20, 20, 299, 53))
        self.layoutWidget.setObjectName("layoutWidget")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.layoutWidget)
        self.gridLayout_4.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.checkBoxDefaultFuzzingPause = QtWidgets.QCheckBox(self.layoutWidget)
        self.checkBoxDefaultFuzzingPause.setObjectName("checkBoxDefaultFuzzingPause")
        self.gridLayout_4.addWidget(self.checkBoxDefaultFuzzingPause, 0, 0, 1, 2)
        self.doubleSpinBoxFuzzingPause = KillerDoubleSpinBox(self.layoutWidget)
        self.doubleSpinBoxFuzzingPause.setDecimals(3)
        self.doubleSpinBoxFuzzingPause.setMaximum(999999999.0)
        self.doubleSpinBoxFuzzingPause.setObjectName("doubleSpinBoxFuzzingPause")
        self.gridLayout_4.addWidget(self.doubleSpinBoxFuzzingPause, 1, 0, 1, 1)
        self.labelFuzzingSamples = QtWidgets.QLabel(self.layoutWidget)
        self.labelFuzzingSamples.setObjectName("labelFuzzingSamples")
        self.gridLayout_4.addWidget(self.labelFuzzingSamples, 1, 1, 1, 1)
        self.tabWidget.addTab(self.tabGeneration, "")
        self.tabView = QtWidgets.QWidget()
        self.tabView.setObjectName("tabView")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.tabView)
        self.verticalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_7 = QtWidgets.QLabel(self.tabView)
        self.label_7.setObjectName("label_7")
        self.horizontalLayout_2.addWidget(self.label_7)
        self.comboBoxDefaultView = QtWidgets.QComboBox(self.tabView)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.comboBoxDefaultView.sizePolicy().hasHeightForWidth())
        self.comboBoxDefaultView.setSizePolicy(sizePolicy)
        self.comboBoxDefaultView.setObjectName("comboBoxDefaultView")
        self.comboBoxDefaultView.addItem("")
        self.comboBoxDefaultView.addItem("")
        self.comboBoxDefaultView.addItem("")
        self.horizontalLayout_2.addWidget(self.comboBoxDefaultView)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.verticalLayout_4.addLayout(self.horizontalLayout_2)
        self.checkBoxShowConfirmCloseDialog = QtWidgets.QCheckBox(self.tabView)
        self.checkBoxShowConfirmCloseDialog.setObjectName("checkBoxShowConfirmCloseDialog")
        self.verticalLayout_4.addWidget(self.checkBoxShowConfirmCloseDialog)
        self.checkBoxHoldShiftToDrag = QtWidgets.QCheckBox(self.tabView)
        self.checkBoxHoldShiftToDrag.setObjectName("checkBoxHoldShiftToDrag")
        self.verticalLayout_4.addWidget(self.checkBoxHoldShiftToDrag)
        self.checkBoxPauseTime = QtWidgets.QCheckBox(self.tabView)
        self.checkBoxPauseTime.setObjectName("checkBoxPauseTime")
        self.verticalLayout_4.addWidget(self.checkBoxPauseTime)
        self.checkBoxAlignLabels = QtWidgets.QCheckBox(self.tabView)
        self.checkBoxAlignLabels.setObjectName("checkBoxAlignLabels")
        self.verticalLayout_4.addWidget(self.checkBoxAlignLabels)
        self.checkBoxFallBackTheme = QtWidgets.QCheckBox(self.tabView)
        self.checkBoxFallBackTheme.setObjectName("checkBoxFallBackTheme")
        self.verticalLayout_4.addWidget(self.checkBoxFallBackTheme)
        spacerItem2 = QtWidgets.QSpacerItem(20, 383, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_4.addItem(spacerItem2)
        self.tabWidget.addTab(self.tabView, "")
        self.tabFieldtypes = QtWidgets.QWidget()
        self.tabFieldtypes.setObjectName("tabFieldtypes")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.tabFieldtypes)
        self.verticalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.tblLabeltypes = QtWidgets.QTableView(self.tabFieldtypes)
        self.tblLabeltypes.setAlternatingRowColors(True)
        self.tblLabeltypes.setObjectName("tblLabeltypes")
        self.horizontalLayout_3.addWidget(self.tblLabeltypes)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.btnAddLabelType = QtWidgets.QToolButton(self.tabFieldtypes)
        icon = QtGui.QIcon.fromTheme("list-add")
        self.btnAddLabelType.setIcon(icon)
        self.btnAddLabelType.setObjectName("btnAddLabelType")
        self.verticalLayout_3.addWidget(self.btnAddLabelType)
        self.btnRemoveLabeltype = QtWidgets.QToolButton(self.tabFieldtypes)
        icon = QtGui.QIcon.fromTheme("list-remove")
        self.btnRemoveLabeltype.setIcon(icon)
        self.btnRemoveLabeltype.setObjectName("btnRemoveLabeltype")
        self.verticalLayout_3.addWidget(self.btnRemoveLabeltype)
        spacerItem3 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem3)
        self.horizontalLayout_3.addLayout(self.verticalLayout_3)
        self.verticalLayout_5.addLayout(self.horizontalLayout_3)
        spacerItem4 = QtWidgets.QSpacerItem(20, 203, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_5.addItem(spacerItem4)
        self.tabWidget.addTab(self.tabFieldtypes, "")
        self.tab_plugins = QtWidgets.QWidget()
        self.tab_plugins.setObjectName("tab_plugins")
        self.tabWidget.addTab(self.tab_plugins, "")
        self.tabDevices = QtWidgets.QWidget()
        self.tabDevices.setObjectName("tabDevices")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.tabDevices)
        self.gridLayout_3.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.listWidgetDevices = QtWidgets.QListWidget(self.tabDevices)
        self.listWidgetDevices.setObjectName("listWidgetDevices")
        self.gridLayout_3.addWidget(self.listWidgetDevices, 0, 0, 3, 2)
        self.chkBoxDeviceEnabled = QtWidgets.QCheckBox(self.tabDevices)
        self.chkBoxDeviceEnabled.setObjectName("chkBoxDeviceEnabled")
        self.gridLayout_3.addWidget(self.chkBoxDeviceEnabled, 0, 2, 1, 1)
        self.rbNativeBackend = QtWidgets.QRadioButton(self.tabDevices)
        self.rbNativeBackend.setObjectName("rbNativeBackend")
        self.gridLayout_3.addWidget(self.rbNativeBackend, 1, 2, 1, 1)
        self.rbGnuradioBackend = QtWidgets.QRadioButton(self.tabDevices)
        self.rbGnuradioBackend.setObjectName("rbGnuradioBackend")
        self.gridLayout_3.addWidget(self.rbGnuradioBackend, 2, 2, 1, 1)
        self.lSupport = QtWidgets.QLabel(self.tabDevices)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lSupport.sizePolicy().hasHeightForWidth())
        self.lSupport.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setItalic(False)
        font.setWeight(75)
        self.lSupport.setFont(font)
        self.lSupport.setStyleSheet("color: green")
        self.lSupport.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.lSupport.setObjectName("lSupport")
        self.gridLayout_3.addWidget(self.lSupport, 3, 0, 1, 2)
        self.line = QtWidgets.QFrame(self.tabDevices)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.gridLayout_3.addWidget(self.line, 5, 0, 1, 3)
        self.label_8 = QtWidgets.QLabel(self.tabDevices)
        self.label_8.setObjectName("label_8")
        self.gridLayout_3.addWidget(self.label_8, 7, 0, 1, 1)
        self.line_2 = QtWidgets.QFrame(self.tabDevices)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.gridLayout_3.addWidget(self.line_2, 9, 0, 1, 3)
        self.groupBox_3 = QtWidgets.QGroupBox(self.tabDevices)
        self.groupBox_3.setObjectName("groupBox_3")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.groupBox_3)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.label_11 = QtWidgets.QLabel(self.groupBox_3)
        font = QtGui.QFont()
        font.setItalic(True)
        self.label_11.setFont(font)
        self.label_11.setObjectName("label_11")
        self.gridLayout_2.addWidget(self.label_11, 0, 0, 1, 2)
        self.lineEditPython2Interpreter = QtWidgets.QLineEdit(self.groupBox_3)
        self.lineEditPython2Interpreter.setObjectName("lineEditPython2Interpreter")
        self.gridLayout_2.addWidget(self.lineEditPython2Interpreter, 1, 1, 1, 1)
        self.lGnuradioInstalled = QtWidgets.QLabel(self.groupBox_3)
        self.lGnuradioInstalled.setStyleSheet("")
        self.lGnuradioInstalled.setObjectName("lGnuradioInstalled")
        self.gridLayout_2.addWidget(self.lGnuradioInstalled, 3, 0, 1, 2)
        self.lineEditGnuradioDirectory = QtWidgets.QLineEdit(self.groupBox_3)
        self.lineEditGnuradioDirectory.setEnabled(True)
        self.lineEditGnuradioDirectory.setObjectName("lineEditGnuradioDirectory")
        self.gridLayout_2.addWidget(self.lineEditGnuradioDirectory, 2, 1, 1, 1)
        self.radioButtonPython2Interpreter = QtWidgets.QRadioButton(self.groupBox_3)
        self.radioButtonPython2Interpreter.setObjectName("radioButtonPython2Interpreter")
        self.gridLayout_2.addWidget(self.radioButtonPython2Interpreter, 1, 0, 1, 1)
        self.radioButtonGnuradioDirectory = QtWidgets.QRadioButton(self.groupBox_3)
        self.radioButtonGnuradioDirectory.setObjectName("radioButtonGnuradioDirectory")
        self.gridLayout_2.addWidget(self.radioButtonGnuradioDirectory, 2, 0, 1, 1)
        self.gridLayout_3.addWidget(self.groupBox_3, 10, 0, 1, 2)
        spacerItem5 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_3.addItem(spacerItem5, 11, 1, 1, 1)
        self.labelWindowsError = QtWidgets.QLabel(self.tabDevices)
        self.labelWindowsError.setWordWrap(True)
        self.labelWindowsError.setObjectName("labelWindowsError")
        self.gridLayout_3.addWidget(self.labelWindowsError, 4, 0, 1, 2)
        self.label_5 = QtWidgets.QLabel(self.tabDevices)
        self.label_5.setObjectName("label_5")
        self.gridLayout_3.addWidget(self.label_5, 8, 0, 1, 1)
        self.doubleSpinBoxRAMThreshold = QtWidgets.QDoubleSpinBox(self.tabDevices)
        self.doubleSpinBoxRAMThreshold.setMinimum(1.0)
        self.doubleSpinBoxRAMThreshold.setMaximum(100.0)
        self.doubleSpinBoxRAMThreshold.setObjectName("doubleSpinBoxRAMThreshold")
        self.gridLayout_3.addWidget(self.doubleSpinBoxRAMThreshold, 8, 1, 1, 2)
        self.spinBoxNumSendingRepeats = QtWidgets.QSpinBox(self.tabDevices)
        self.spinBoxNumSendingRepeats.setProperty("showGroupSeparator", False)
        self.spinBoxNumSendingRepeats.setMaximum(999999999)
        self.spinBoxNumSendingRepeats.setDisplayIntegerBase(10)
        self.spinBoxNumSendingRepeats.setObjectName("spinBoxNumSendingRepeats")
        self.gridLayout_3.addWidget(self.spinBoxNumSendingRepeats, 7, 1, 1, 2)
        self.tabWidget.addTab(self.tabDevices, "")
        self.horizontalLayout.addWidget(self.tabWidget)

        self.retranslateUi(DialogOptions)
        self.tabWidget.setCurrentIndex(5)
        QtCore.QMetaObject.connectSlotsByName(DialogOptions)

    def retranslateUi(self, DialogOptions):
        _translate = QtCore.QCoreApplication.translate
        DialogOptions.setWindowTitle(_translate("DialogOptions", "Options"))
        self.groupBox.setTitle(_translate("DialogOptions", "Symbols"))
        self.label.setText(_translate("DialogOptions", "Some protocols use different information lengths. This can be part of the protocol logic (e.g. to indicate a SOF). You can set a tolerance window for the selected bit length, outside the window a new symbol will be created."))
        self.label_2.setText(_translate("DialogOptions", "Tolerance window:"))
        self.spinBoxSymbolTreshold.setSuffix(_translate("DialogOptions", "%"))
        self.label_3.setText(_translate("DialogOptions", "of selected bit length"))
        self.label_4.setText(_translate("DialogOptions", "Relative symbol length:"))
        self.lSymbolLength.setText(_translate("DialogOptions", "0%"))
        self.label_6.setText(_translate("DialogOptions", "of selected bit length"))
        self.chkBoxEnableSymbols.setText(_translate("DialogOptions", "Enable symbols"))
        self.lExplanation.setText(_translate("DialogOptions", "No Symbols will be created"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tabInterpretation), _translate("DialogOptions", "Interpretation"))
        self.checkBoxDefaultFuzzingPause.setToolTip(_translate("DialogOptions", "<html><head/><body><p>If you disable the default pause, the pause of the fuzzed message will be used.</p></body></html>"))
        self.checkBoxDefaultFuzzingPause.setText(_translate("DialogOptions", "Use a default pause for fuzzed messages"))
        self.labelFuzzingSamples.setText(_translate("DialogOptions", "Samples"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tabGeneration), _translate("DialogOptions", "Generation"))
        self.label_7.setText(_translate("DialogOptions", "Default View:"))
        self.comboBoxDefaultView.setItemText(0, _translate("DialogOptions", "Bit"))
        self.comboBoxDefaultView.setItemText(1, _translate("DialogOptions", "Hex"))
        self.comboBoxDefaultView.setItemText(2, _translate("DialogOptions", "ASCII"))
        self.checkBoxShowConfirmCloseDialog.setText(_translate("DialogOptions", "Show \"confirm close\" dialog"))
        self.checkBoxHoldShiftToDrag.setText(_translate("DialogOptions", "Hold shift to drag"))
        self.checkBoxPauseTime.setText(_translate("DialogOptions", "Show pauses as time"))
        self.checkBoxAlignLabels.setText(_translate("DialogOptions", "Align on labels"))
        self.checkBoxFallBackTheme.setToolTip(_translate("DialogOptions", "Tick this option if you experience problems with you current Qt theme like no colors in table headers."))
        self.checkBoxFallBackTheme.setText(_translate("DialogOptions", "Use fallback application theme [RESTART REQUIRED]"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tabView), _translate("DialogOptions", "View"))
        self.btnAddLabelType.setText(_translate("DialogOptions", "..."))
        self.btnRemoveLabeltype.setText(_translate("DialogOptions", "..."))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tabFieldtypes), _translate("DialogOptions", "Fieldtypes"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_plugins), _translate("DialogOptions", "Plugins"))
        self.chkBoxDeviceEnabled.setText(_translate("DialogOptions", "Enabled"))
        self.rbNativeBackend.setText(_translate("DialogOptions", "Native backend (recommended)"))
        self.rbGnuradioBackend.setText(_translate("DialogOptions", "Gnuradio backend"))
        self.lSupport.setText(_translate("DialogOptions", "device supports sending and receiving"))
        self.label_8.setText(_translate("DialogOptions", "Default sending repititions:"))
        self.groupBox_3.setTitle(_translate("DialogOptions", "Gnuradio options"))
        self.label_11.setText(_translate("DialogOptions", "Needed for Gnuradio backend only"))
        self.lineEditPython2Interpreter.setToolTip(_translate("DialogOptions", "<html><head/><body><p>Use this option if you installed Gnuradio with your package manager e.g. on Linux and Mac OS X.</p></body></html>"))
        self.lineEditPython2Interpreter.setPlaceholderText(_translate("DialogOptions", "/usr/bin/python2"))
        self.lGnuradioInstalled.setText(_translate("DialogOptions", "Gnuradio installation found"))
        self.lineEditGnuradioDirectory.setToolTip(_translate("DialogOptions", "<html><head/><body><p>If you installed Gnuradio with a bundled python interpreter, you need to enter the site-packages path of the installation here. The path should be something like <span style=\" font-style:italic;\">C:\\Program Files\\GNURadio-3.7</span>.</p></body></html>"))
        self.lineEditGnuradioDirectory.setPlaceholderText(_translate("DialogOptions", "C:\\...\\Gnuradio"))
        self.radioButtonPython2Interpreter.setToolTip(_translate("DialogOptions", "<html><head/><body><p>Use this option if you installed Gnuradio with your package manager e.g. on Linux and Mac OS X.</p></body></html>"))
        self.radioButtonPython2Interpreter.setText(_translate("DialogOptions", "Python2 interpreter"))
        self.radioButtonGnuradioDirectory.setToolTip(_translate("DialogOptions", "<html><head/><body><p>If you installed Gnuradio with a bundled python interpreter, you need to enter the site-packages path of the installation here. The path should be something like <span style=\" font-style:italic;\">C:\\Program Files\\GNURadio-3.7</span>.</p></body></html>"))
        self.radioButtonGnuradioDirectory.setText(_translate("DialogOptions", "Gnuradio Directory"))
        self.labelWindowsError.setText(_translate("DialogOptions", "<html><head/><body><p><span style=\" color:#ff0000;\">Detected a 32 bit installation of python 3.</span> Install <span style=\" font-weight:600;\">64 bit version</span> to use native backends.</p></body></html>"))
        self.label_5.setText(_translate("DialogOptions", "Use this percentage of available RAM for buffer allocation:"))
        self.doubleSpinBoxRAMThreshold.setSuffix(_translate("DialogOptions", "%"))
        self.spinBoxNumSendingRepeats.setSpecialValueText(_translate("DialogOptions", "Infinite"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tabDevices), _translate("DialogOptions", "Device"))

from urh.ui.KillerDoubleSpinBox import KillerDoubleSpinBox



from PyQt5.QtCore import Qt
from PyQt5.QtGui import QKeyEvent
from PyQt5.QtWidgets import QTableView, QMenu

from urh.models.PLabelTableModel import PLabelTableModel


class ProtocolLabelTableView(QTableView):
    def __init__(self, parent=None):
        super().__init__(parent)

    def model(self) -> PLabelTableModel:
        return super().model()

    def contextMenuEvent(self, event):
        menu = QMenu()
        pos = event.pos()
        row = self.rowAt(pos.y())
        if row != -1:
            delAction = menu.addAction("Delete Protocol Label")
            action = menu.exec_(self.mapToGlobal(pos))
            if action == delAction:
                lbl = self.model().proto_analyzer.protocol_labels[row]
                self.model().remove_label(lbl)

    def keyPressEvent(self, event: QKeyEvent):
        if event.key() == Qt.Key_Delete:
            # Delete selected rows
            rows = [i.row() for i in self.selectedIndexes()]
            for row in sorted(rows, reverse=True):
                self.model().remove_label_at(row)
        else:
            event.accept()


from subprocess import call, Popen

MODULES = ["path_creator", "signalFunctions", "util"]

for module in MODULES:
    call(["cython", "-a", "--cplus", "-3", module + ".pyx"])
    Popen(["google-chrome-stable", module + ".html"])
